import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

// Exemple de validar arxius XML + DTD

public class XMLValidator {

  private XMLValidator() {}
   
  // validate using DOM (DTD as defined in the XML)
  public static boolean validateWithDTDUsingDOM(String xmlPath) throws ParserConfigurationException, IOException {
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating(true);
      factory.setNamespaceAware(true);

      DocumentBuilder builder = factory.newDocumentBuilder();

      builder.setErrorHandler(
          new ErrorHandler() {
            public void warning(SAXParseException e) throws SAXException {
              System.out.println("WARNING : " + e.getMessage());
            }

            public void error(SAXParseException e) throws SAXException {
              System.out.println("ERROR : " + e.getMessage());
              throw e;
            }

            public void fatalError(SAXParseException e) throws SAXException {
              System.out.println("FATAL : " + e.getMessage());
              throw e;
            }
          }
      );

      builder.parse(new File(xmlPath));
      return true;
    }
    catch (ParserConfigurationException pce) { throw pce; } 
    catch (IOException io) { throw io; }
    catch (SAXException se){ return false; }
  }

  
  public static void main (String args[]) throws Exception{ 
    String basePath = System.getProperty("user.dir"); // + "/FactoryOrdinador";
    String path = basePath + "/school.xml";

    boolean isValid = XMLValidator.validateWithDTDUsingDOM(path);
    if (isValid) {
        System.out.println("XML and DTD are valid");
    } else {
        System.out.println("XML or DTD are NOT valid");
    }
  }
}